@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2018 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -17,6 +17,13 @@
  *********************************************************/
 
 
+#if defined(USE_GOBUILD_TOOLS_VERSION) && !defined(GOBUILD_TOOLS_VERSION_USED)
+
+#define GOBUILD_TOOLS_VERSION_USED
+#include "vmtools_version.h"
+
+#else
+
 #ifndef VM_TOOLS_VERSION_H
 #define VM_TOOLS_VERSION_H
 
@@ -62,6 +69,10 @@
 
 #ifndef RC_INVOKED
 
+#if defined __cplusplus
+extern "C" {
+#endif
+
 typedef uint32 ToolsVersion;
 typedef enum {
   TOOLS_TYPE_UNSET = -1,
@@ -87,8 +98,13 @@ TOOLS_VERSION_UINT_TO_COMPONENTS(const ToolsVersion toolsVersion,   // IN
    comps->minor = (toolsVersion >> 5) & 0x1f;
    comps->base  = toolsVersion & 0x1f;
 }
+
+#if defined __cplusplus
+} // extern "C"
 #endif
 
+#endif // RC_INVOKED
+
 /*
  * Reserve the highest possible Tools version for Tools whose lifecycle isn't
  * to be managed by VMware's platform.
@@ -1340,22 +1356,141 @@ TOOLS_VERSION_UINT_TO_COMPONENTS(const ToolsVersion toolsVersion,   // IN
 #define   TOOLS_VERSION_TORQUE_WRENCH_RELEASE_V_BASE 0
 
 #ifndef RC_INVOKED
-#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE1  TOOLS_VERSION_TO_UINT(TOOLS_VERSION_TORQUE_WRENCH_UPDATE1_V)
+#define   TOOLS_VERSION_TORQUE_WRENCH_PATCH1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_TORQUE_WRENCH_PATCH1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_TORQUE_WRENCH_PATCH1_V_MJR  10
+#define   TOOLS_VERSION_TORQUE_WRENCH_PATCH1_V_MNR  0
+#define   TOOLS_VERSION_TORQUE_WRENCH_PATCH1_V_BASE 1
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_TORQUE_WRENCH_UPDATE1_V)
 #endif /* RC_INVOKED */
 #define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE1_V_MJR  10
 #define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE1_V_MNR  0
 #define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE1_V_BASE 5
 
 #ifndef RC_INVOKED
-#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE3  TOOLS_VERSION_TO_UINT(TOOLS_VERSION_TORQUE_WRENCH_UPDATE3_V)
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE2 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_TORQUE_WRENCH_UPDATE2_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE2_V_MJR  10
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE2_V_MNR  0
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE2_V_BASE 6
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE3OVT TOOLS_VERSION_TO_UINT(TOOLS_VERSION_TORQUE_WRENCH_UPDATE3OVT_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE3OVT_V_MJR  10
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE3OVT_V_MNR  0
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE3OVT_V_BASE 7
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE3 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_TORQUE_WRENCH_UPDATE3_V)
 #endif /* RC_INVOKED */
 #define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE3_V_MJR  10
 #define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE3_V_MNR  0
-#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE3_V_BASE 7
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE3_V_BASE 8
 
-#define   TOOLS_VERSION_CURRENT        TOOLS_VERSION_TORQUE_WRENCH_UPDATE3
-#define   TOOLS_VERSION_CURRENT_STR    TOOLS_VERSION_TO_STR(TOOLS_VERSION_TORQUE_WRENCH_UPDATE3)
-#define   TOOLS_VERSION_CURRENT_CSV    TOOLS_VERSION_TO_CSV(TOOLS_VERSION_TORQUE_WRENCH_UPDATE3)
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE4 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_TORQUE_WRENCH_UPDATE4_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE4_V_MJR  10
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE4_V_MNR  0
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE4_V_BASE 9
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE5  TOOLS_VERSION_TO_UINT(TOOLS_VERSION_TORQUE_WRENCH_UPDATE5_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE5_V_MJR  10
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE5_V_MNR  0
+#define   TOOLS_VERSION_TORQUE_WRENCH_UPDATE5_V_BASE 10
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_TORQUE_WRENCH_FROZEN1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_TORQUE_WRENCH_FROZEN1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_TORQUE_WRENCH_FROZEN1_V_MJR  10
+#define   TOOLS_VERSION_TORQUE_WRENCH_FROZEN1_V_MNR  0
+#define   TOOLS_VERSION_TORQUE_WRENCH_FROZEN1_V_BASE 12
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_SCREW_DRIVER_RELEASE TOOLS_VERSION_TO_UINT(TOOLS_VERSION_SCREW_DRIVER_RELEASE_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_SCREW_DRIVER_RELEASE_V_MJR  10
+#define   TOOLS_VERSION_SCREW_DRIVER_RELEASE_V_MNR  1
+#define   TOOLS_VERSION_SCREW_DRIVER_RELEASE_V_BASE 0
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_SCREW_DRIVER_UPDATE1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_SCREW_DRIVER_UPDATE1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_SCREW_DRIVER_UPDATE1_V_MJR  10
+#define   TOOLS_VERSION_SCREW_DRIVER_UPDATE1_V_MNR  1
+#define   TOOLS_VERSION_SCREW_DRIVER_UPDATE1_V_BASE 5
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_SCREW_DRIVER_PATCH2  TOOLS_VERSION_TO_UINT(TOOLS_VERSION_SCREW_DRIVER_PATCH2_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_SCREW_DRIVER_PATCH2_V_MJR  10
+#define   TOOLS_VERSION_SCREW_DRIVER_PATCH2_V_MNR  1
+#define   TOOLS_VERSION_SCREW_DRIVER_PATCH2_V_BASE 6
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_SCREW_DRIVER_UPDATE2 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_SCREW_DRIVER_UPDATE2_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_SCREW_DRIVER_UPDATE2_V_MJR  10
+#define   TOOLS_VERSION_SCREW_DRIVER_UPDATE2_V_MNR  1
+#define   TOOLS_VERSION_SCREW_DRIVER_UPDATE2_V_BASE 10
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_SCREW_DRIVER_UPDATE3 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_SCREW_DRIVER_UPDATE3_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_SCREW_DRIVER_UPDATE3_V_MJR  10
+#define   TOOLS_VERSION_SCREW_DRIVER_UPDATE3_V_MNR  1
+#define   TOOLS_VERSION_SCREW_DRIVER_UPDATE3_V_BASE 15
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_CHAINSAW_RELEASE TOOLS_VERSION_TO_UINT(TOOLS_VERSION_CHAINSAW_RELEASE_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_CHAINSAW_RELEASE_V_MJR  10
+#define   TOOLS_VERSION_CHAINSAW_RELEASE_V_MNR  2
+#define   TOOLS_VERSION_CHAINSAW_RELEASE_V_BASE 0
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_CHAINSAW_PATCH1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_CHAINSAW_PATCH1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_CHAINSAW_PATCH1_V_MJR  10
+#define   TOOLS_VERSION_CHAINSAW_PATCH1_V_MNR  2
+#define   TOOLS_VERSION_CHAINSAW_PATCH1_V_BASE 1
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_CHAINSAW_UPDATE1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_CHAINSAW_UPDATE1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_CHAINSAW_UPDATE1_V_MJR  10
+#define   TOOLS_VERSION_CHAINSAW_UPDATE1_V_MNR  2
+#define   TOOLS_VERSION_CHAINSAW_UPDATE1_V_BASE 5
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_CHAINSAW_PATCH2 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_CHAINSAW_PATCH2_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_CHAINSAW_PATCH2_V_MJR  10
+#define   TOOLS_VERSION_CHAINSAW_PATCH2_V_MNR  2
+#define   TOOLS_VERSION_CHAINSAW_PATCH2_V_BASE 6
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_JACKHAMMER_RELEASE TOOLS_VERSION_TO_UINT(TOOLS_VERSION_JACKHAMMER_RELEASE_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_JACKHAMMER_RELEASE_V_MJR  10
+#define   TOOLS_VERSION_JACKHAMMER_RELEASE_V_MNR  3
+#define   TOOLS_VERSION_JACKHAMMER_RELEASE_V_BASE 0
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_JACKHAMMER_UPDATE1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_JACKHAMMER_UPDATE1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_JACKHAMMER_UPDATE1_V_MJR  10
+#define   TOOLS_VERSION_JACKHAMMER_UPDATE1_V_MNR  3
+#define   TOOLS_VERSION_JACKHAMMER_UPDATE1_V_BASE 5
+
+#define   TOOLS_VERSION_CURRENT        TOOLS_VERSION_JACKHAMMER_UPDATE1
+#define   TOOLS_VERSION_CURRENT_STR    TOOLS_VERSION_TO_STR(TOOLS_VERSION_JACKHAMMER_UPDATE1)
+#define   TOOLS_VERSION_CURRENT_CSV    TOOLS_VERSION_TO_CSV(TOOLS_VERSION_JACKHAMMER_UPDATE1)
 
 /*
  * The extended Tools version is the current Tools version with the
@@ -1373,3 +1508,5 @@ TOOLS_VERSION_UINT_TO_COMPONENTS(const ToolsVersion toolsVersion,   // IN
    TOOLS_VERSION_CURRENT_STR "." PRODUCT_BUILD_NUMBER_NUMERIC_STRING
 
 #endif /* VM_TOOLS_VERSION_H */
+
+#endif /* defined(USE_GOBUILD_TOOLS_VERSION) && !defined(GOBUILD_TOOLS_VERSION_USED) */
