@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2007-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2007-2018 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -35,7 +35,6 @@
 
 #include "vmBackupInt.h"
 
-#include "dynxdr.h"
 #include <glib-object.h>
 #include <gmodule.h>
 #include "guestApp.h"
@@ -74,6 +73,20 @@ VM_EMBED_VERSION(VMTOOLSD_VERSION_STRING);
                             NULL);                                            \
 } while (0)
 
+/*
+ * Macros to read values from config file.
+ */
+#define VMBACKUP_CONFIG_GET_BOOL(config, key, defVal)       \
+   VMTools_ConfigGetBoolean(config, "vmbackup", key, defVal)
+
+#define VMBACKUP_CONFIG_GET_STR(config, key, defVal)        \
+   VMTools_ConfigGetString(config, "vmbackup", key, defVal)
+
+#define VMBACKUP_CONFIG_GET_INT(config, key, defVal)        \
+   VMTools_ConfigGetInteger(config, "vmbackup", key, defVal)
+
+#define VMBACKUP_CFG_ENABLEVSS      "enableVSS"
+
 static VmBackupState *gBackupState = NULL;
 
 static Bool
@@ -82,6 +95,9 @@ VmBackupEnableSync(void);
 static Bool
 VmBackupEnableSyncWait(void);
 
+static Bool
+VmBackupEnableCompleteWait(void);
+
 
 /**
  * Returns a string representation of the given state machine state.
@@ -113,6 +129,9 @@ VmBackupGetStateName(VmBackupMState state)
    case VMBACKUP_MSTATE_SCRIPT_THAW:
       return "SCRIPT_THAW";
 
+   case VMBACKUP_MSTATE_COMPLETE_WAIT:
+      return "COMPLETE_WAIT";
+
    case VMBACKUP_MSTATE_SCRIPT_ERROR:
       return "SCRIPT_ERROR";
 
@@ -249,7 +268,8 @@ VmBackup_SendEvent(const char *event,
 #endif
 
    if (!success) {
-      g_warning("Failed to send vmbackup event: %s.\n", result);
+      g_warning("Failed to send vmbackup event: %s, result: %s.\n",
+                msg, result);
    }
    vm_free(result);
    g_free(msg);
@@ -299,10 +319,14 @@ VmBackupFinalize(void)
    }
 
    gBackupState->provider->release(gBackupState->provider);
+   if (gBackupState->completer != NULL) {
+      gBackupState->completer->release(gBackupState->completer);
+   }
    g_static_mutex_free(&gBackupState->opLock);
    g_free(gBackupState->scriptArg);
    g_free(gBackupState->volumes);
    g_free(gBackupState->snapshots);
+   g_free(gBackupState->excludedFileSystems);
    g_free(gBackupState->errorMsg);
    g_free(gBackupState);
    gBackupState = NULL;
@@ -393,6 +417,7 @@ VmBackupOnError(void)
       break;
 
    case VMBACKUP_MSTATE_SCRIPT_THAW:
+   case VMBACKUP_MSTATE_COMPLETE_WAIT:
       /* Next state is "idle". */
       gBackupState->machineState = VMBACKUP_MSTATE_IDLE;
       break;
@@ -417,6 +442,7 @@ VmBackupDoAbort(void)
    ASSERT(gBackupState != NULL);
    if (gBackupState->machineState != VMBACKUP_MSTATE_SCRIPT_ERROR &&
        gBackupState->machineState != VMBACKUP_MSTATE_SYNC_ERROR) {
+      const char *eventMsg = "Quiesce aborted.";
       /* Mark the current operation as cancelled. */
       g_static_mutex_lock(&gBackupState->opLock);
       if (gBackupState->currentOp != NULL) {
@@ -426,9 +452,21 @@ VmBackupDoAbort(void)
       }
       g_static_mutex_unlock(&gBackupState->opLock);
 
+#ifdef __linux__
+      /* Thaw the guest if already quiesced */
+      if (gBackupState->machineState == VMBACKUP_MSTATE_SYNC_FREEZE) {
+         g_debug("Guest already quiesced, thawing for abort\n");
+         if (!gBackupState->provider->snapshotDone(gBackupState,
+                                      gBackupState->provider->clientData)) {
+            g_debug("Thaw during abort failed\n");
+            eventMsg = "Quiesce could not be aborted.";
+         }
+      }
+#endif
+
       VmBackup_SendEvent(VMBACKUP_EVENT_REQUESTOR_ABORT,
                          VMBACKUP_REMOTE_ABORT,
-                         "Quiesce aborted.");
+                         eventMsg);
 
       /* Transition to the error state. */
       if (VmBackupOnError()) {
@@ -459,51 +497,49 @@ VmBackupAbortTimer(gpointer data)
 
 
 /**
- * Callback that checks for the status of the current operation. Calls the
- * queued operations as needed.
+ * Post-process the current operation. Calls the queued
+ * operations as needed.
  *
- * @param[in]  clientData     Unused.
+ * @param[out]    pending     Tells if currentOp is pending.
  *
- * @return FALSE
+ * @return TRUE if currentOp finished or pending, FALSE on error.
  */
 
 static gboolean
-VmBackupAsyncCallback(void *clientData)
+VmBackupPostProcessCurrentOp(gboolean *pending)
 {
+   gboolean retVal = TRUE;
    VmBackupOpStatus status = VMBACKUP_STATUS_FINISHED;
 
    g_debug("*** %s\n", __FUNCTION__);
    ASSERT(gBackupState != NULL);
 
-   g_source_unref(gBackupState->timerEvent);
-   gBackupState->timerEvent = NULL;
+   *pending = FALSE;
 
    g_static_mutex_lock(&gBackupState->opLock);
+
    if (gBackupState->currentOp != NULL) {
-      g_debug("VmBackupAsyncCallback: checking %s\n", gBackupState->currentOpName);
+      g_debug("%s: checking %s\n", __FUNCTION__, gBackupState->currentOpName);
       status = VmBackup_QueryStatus(gBackupState->currentOp);
    }
-   g_static_mutex_unlock(&gBackupState->opLock);
 
    switch (status) {
    case VMBACKUP_STATUS_PENDING:
+      *pending = TRUE;
       goto exit;
 
    case VMBACKUP_STATUS_FINISHED:
-      g_static_mutex_lock(&gBackupState->opLock);
       if (gBackupState->currentOpName != NULL) {
          g_debug("Async request '%s' completed\n", gBackupState->currentOpName);
          VmBackup_Release(gBackupState->currentOp);
          gBackupState->currentOpName = NULL;
       }
       gBackupState->currentOp = NULL;
-      g_static_mutex_unlock(&gBackupState->opLock);
       break;
 
    default:
       {
          gchar *msg;
-         g_static_mutex_lock(&gBackupState->opLock);
          if (gBackupState->errorMsg != NULL) {
             msg = g_strdup_printf("'%s' operation failed: %s",
                                   gBackupState->currentOpName,
@@ -519,8 +555,7 @@ VmBackupAsyncCallback(void *clientData)
 
          VmBackup_Release(gBackupState->currentOp);
          gBackupState->currentOp = NULL;
-         g_static_mutex_unlock(&gBackupState->opLock);
-         VmBackupOnError();
+         retVal = FALSE;
          goto exit;
       }
    }
@@ -533,23 +568,62 @@ VmBackupAsyncCallback(void *clientData)
       VmBackupCallback cb = gBackupState->callback;
       gBackupState->callback = NULL;
 
+      /*
+       * Callback should not acquire opLock, but instead assume that
+       * it holds the lock already.
+       */
       if (cb(gBackupState)) {
-         g_static_mutex_lock(&gBackupState->opLock);
          if (gBackupState->currentOp != NULL || gBackupState->forceRequeue) {
-            g_static_mutex_unlock(&gBackupState->opLock);
             goto exit;
          }
-         g_static_mutex_unlock(&gBackupState->opLock);
       } else {
-         VmBackupOnError();
+         retVal = FALSE;
          goto exit;
       }
    }
 
+exit:
+   g_static_mutex_unlock(&gBackupState->opLock);
+   return retVal;
+}
+
+
+/**
+ * Callback to advance the state machine to next state once
+ * current operation finishes.
+ *
+ * @param[in]  clientData     Unused.
+ *
+ * @return FALSE
+ */
+
+static gboolean
+VmBackupAsyncCallback(void *clientData)
+{
+   gboolean opPending;
+   g_debug("*** %s\n", __FUNCTION__);
+   ASSERT(gBackupState != NULL);
+
+   g_source_unref(gBackupState->timerEvent);
+   gBackupState->timerEvent = NULL;
+
    /*
-    * At this point, the current operation can be declared finished, and the
-    * state machine can move to the next state.
+    * Move the state machine to the next state, if the
+    * currentOp has finished.
     */
+   if (!VmBackupPostProcessCurrentOp(&opPending)) {
+      VmBackupOnError();
+      goto exit;
+   } else {
+      /*
+       * State transition can't be done if currentOp
+       * has not finished yet.
+       */
+      if (opPending) {
+         goto exit;
+      }
+   }
+
    switch (gBackupState->machineState) {
    case VMBACKUP_MSTATE_SCRIPT_FREEZE:
       /* Next state is "sync freeze wait". */
@@ -583,8 +657,15 @@ VmBackupAsyncCallback(void *clientData)
       }
       break;
 
-   case VMBACKUP_MSTATE_SCRIPT_ERROR:
    case VMBACKUP_MSTATE_SCRIPT_THAW:
+      /* Next state is "complete wait" or "idle". */
+      if (!VmBackupEnableCompleteWait()) {
+         VmBackupOnError();
+      }
+      break;
+
+   case VMBACKUP_MSTATE_SCRIPT_ERROR:
+   case VMBACKUP_MSTATE_COMPLETE_WAIT:
       /* Next state is "idle". */
       gBackupState->machineState = VMBACKUP_MSTATE_IDLE;
       break;
@@ -675,6 +756,62 @@ VmBackupEnableSyncWait(void)
 }
 
 
+/**
+ * Calls the completer's start function and moves the state
+ * machine to next state.
+ *
+ * @return Whether the completer's start callback was successful.
+ */
+
+static Bool
+VmBackupEnableCompleteWait(void)
+{
+   Bool ret = TRUE;
+
+   g_debug("*** %s\n", __FUNCTION__);
+
+   if (gBackupState->completer == NULL) {
+      gBackupState->machineState = VMBACKUP_MSTATE_IDLE;
+      goto exit;
+   }
+
+   if (gBackupState->abortTimer != NULL) {
+      g_source_destroy(gBackupState->abortTimer);
+      g_source_unref(gBackupState->abortTimer);
+
+      /* Host make timeout maximum as 15 minutes for complete process. */
+      if (gBackupState->timeout > GUEST_QUIESCE_DEFAULT_TIMEOUT_IN_SEC) {
+         gBackupState->timeout = GUEST_QUIESCE_DEFAULT_TIMEOUT_IN_SEC;
+      }
+
+      if (gBackupState->timeout != 0) {
+         g_debug("Using completer timeout: %u\n", gBackupState->timeout);
+         gBackupState->abortTimer =
+            g_timeout_source_new_seconds(gBackupState->timeout);
+         VMTOOLSAPP_ATTACH_SOURCE(gBackupState->ctx,
+            gBackupState->abortTimer,
+            VmBackupAbortTimer,
+            NULL,
+            NULL);
+      }
+   }
+
+   if (gBackupState->completer->start(gBackupState,
+                                      gBackupState->completer->clientData)) {
+      /* Move to next state */
+      gBackupState->machineState = VMBACKUP_MSTATE_COMPLETE_WAIT;
+   } else {
+      VmBackup_SendEvent(VMBACKUP_EVENT_REQUESTOR_ERROR,
+                         VMBACKUP_SYNC_ERROR,
+                         "Error when enabling the sync provider.");
+      ret = FALSE;
+   }
+
+exit:
+   return ret;
+}
+
+
 /**
  * After sync provider's start function returns and moves the state
  * machine to VMBACKUP_MSTATE_SYNC_FREEZE state.
@@ -708,36 +845,6 @@ VmBackupEnableSync(void)
 }
 
 
-/**
- * Get boolean entry for the key from the config file.
- *
- * @param[in]  config        Config file to read the key from.
- * @param[in]  key           Key to look for in the config file.
- * @param[in]  defaultValue  Default value if the key is not found.
- */
-
-static gboolean
-VmBackupConfigGetBoolean(GKeyFile *config,
-                         const char *key,
-                         gboolean defValue)
-{
-   GError *err = NULL;
-   gboolean value = defValue;
-
-   if (key != NULL) {
-      value = g_key_file_get_boolean(config,
-                                     "vmbackup",
-                                     key,
-                                     &err);
-      if (err != NULL) {
-         g_clear_error(&err);
-         value = defValue;
-      }
-   }
-   return value;
-}
-
-
 /**
  * Starts the quiesce operation according to the supplied specification unless
  * some unexpected error occurs.
@@ -754,9 +861,9 @@ static gboolean
 VmBackupStartCommon(RpcInData *data,
                     gboolean forceQuiesce)
 {
-   GError *err = NULL;
    ToolsAppCtx *ctx = data->appCtx;
    VmBackupSyncProvider *provider = NULL;
+   VmBackupSyncCompleter *completer = NULL;
 
    size_t i;
 
@@ -766,7 +873,7 @@ VmBackupStartCommon(RpcInData *data,
       const gchar *cfgEntry;
    } providers[] = {
 #if defined(_WIN32)
-      { VmBackup_NewVssProvider, "enableVSS" },
+      { VmBackup_NewVssProvider, VMBACKUP_CFG_ENABLEVSS},
 #endif
       { VmBackup_NewSyncDriverProvider, "enableSyncDriver" },
       { VmBackup_NewNullProvider, NULL },
@@ -779,42 +886,64 @@ VmBackupStartCommon(RpcInData *data,
           * only allow VSS provider
           */
 #if defined(_WIN32)
-         if (VmBackupConfigGetBoolean(ctx->config, "enableVSS", TRUE)) {
+         if (VMBACKUP_CONFIG_GET_BOOL(ctx->config,
+                                      VMBACKUP_CFG_ENABLEVSS, TRUE)) {
             provider = VmBackup_NewVssProvider();
+            if (provider != NULL) {
+               completer = VmBackup_NewVssCompleter(provider);
+               if (completer == NULL) {
+                  g_warning("VSS completion helper cannot be initialized.");
+                  provider->release(provider);
+                  provider = NULL;
+               }
+            }
          }
-#elif defined(_LINUX) || defined(__linux__)
+#elif defined(__linux__)
          /*
           * If quiescing is requested on linux platform,
           * only allow SyncDriver provider
           */
          if (gBackupState->quiesceFS &&
-             VmBackupConfigGetBoolean(ctx->config, "enableSyncDriver", TRUE)) {
+             VMBACKUP_CONFIG_GET_BOOL(ctx->config, "enableSyncDriver", TRUE)) {
             provider = VmBackup_NewSyncDriverOnlyProvider();
-          }
+         }
 #endif
       } else {
          /* If no quiescing is requested only allow null provider */
          provider = VmBackup_NewNullProvider();
       }
-      if (provider == NULL) {
-         g_warning("Requested quiescing cannot be initialized.");
-         goto error;
-      }
    } else {
       /* Instantiate the sync provider. */
       for (i = 0; i < ARRAYSIZE(providers); i++) {
          struct SyncProvider *sp = &providers[i];
 
-         if (VmBackupConfigGetBoolean(ctx->config, sp->cfgEntry, TRUE)) {
+         if (VMBACKUP_CONFIG_GET_BOOL(ctx->config, sp->cfgEntry, TRUE)) {
             provider = sp->ctor();
             if (provider != NULL) {
+#if defined(_WIN32)
+               if (sp->cfgEntry != NULL &&
+                   Str_Strcmp(sp->cfgEntry, VMBACKUP_CFG_ENABLEVSS) == 0) {
+                  completer = VmBackup_NewVssCompleter(provider);
+                  if (completer == NULL) {
+                     g_warning("VSS completion helper cannot be initialized.");
+                     provider->release(provider);
+                     provider = NULL;
+                     continue;
+                  }
+                  break;
+               }
+#else
                break;
+#endif
             }
          }
       }
    }
 
-   ASSERT(provider != NULL);
+   if (provider == NULL) {
+      g_warning("Requested quiescing cannot be initialized.");
+      goto error;
+   }
 
    /* Instantiate the backup state and start the operation. */
    gBackupState->ctx = data->appCtx;
@@ -822,9 +951,10 @@ VmBackupStartCommon(RpcInData *data,
    gBackupState->machineState = VMBACKUP_MSTATE_IDLE;
    gBackupState->freezeStatus = VMBACKUP_FREEZE_FINISHED;
    gBackupState->provider = provider;
+   gBackupState->completer = completer;
    gBackupState->needsPriv = FALSE;
    g_static_mutex_init(&gBackupState->opLock);
-   gBackupState->enableNullDriver = VmBackupConfigGetBoolean(ctx->config,
+   gBackupState->enableNullDriver = VMBACKUP_CONFIG_GET_BOOL(ctx->config,
                                                              "enableNullDriver",
                                                              TRUE);
 
@@ -835,6 +965,13 @@ VmBackupStartCommon(RpcInData *data,
            gBackupState->allowHWProvider, gBackupState->execScripts,
            (gBackupState->scriptArg != NULL) ? gBackupState->scriptArg : "",
            gBackupState->timeout, gBackupState->enableNullDriver, forceQuiesce);
+#if defined(__linux__)
+   gBackupState->excludedFileSystems =
+         VMBACKUP_CONFIG_GET_STR(ctx->config, "excludedFileSystems", NULL);
+   g_debug("Using excludedFileSystems = \"%s\"\n",
+           (gBackupState->excludedFileSystems != NULL) ?
+            gBackupState->excludedFileSystems : "(null)");
+#endif
    g_debug("Quiescing volumes: %s",
            (gBackupState->volumes) ? gBackupState->volumes : "(null)");
 
@@ -864,15 +1001,8 @@ VmBackupStartCommon(RpcInData *data,
     * See bug 506106.
     */
    if (gBackupState->timeout == 0) {
-      gBackupState->timeout = (guint) g_key_file_get_integer(
-                                               gBackupState->ctx->config,
-                                               "vmbackup",
-                                               "timeout",
-                                               &err);
-      if (err != NULL) {
-         g_clear_error(&err);
-         gBackupState->timeout = 15 * 60;
-      }
+      gBackupState->timeout = VMBACKUP_CONFIG_GET_INT(ctx->config, "timeout",
+                                       GUEST_QUIESCE_DEFAULT_TIMEOUT_IN_SEC);
    }
 
    /* Treat "0" as no timeout. */
@@ -898,6 +1028,9 @@ error:
    if (gBackupState->provider) {
       gBackupState->provider->release(gBackupState->provider);
    }
+   if (gBackupState->completer) {
+      gBackupState->completer->release(gBackupState->completer);
+   }
    g_free(gBackupState->scriptArg);
    g_free(gBackupState->volumes);
    g_free(gBackupState);
@@ -923,6 +1056,8 @@ error:
 static gboolean
 VmBackupStart(RpcInData *data)
 {
+   ToolsAppCtx *ctx = data->appCtx;
+
    g_debug("*** %s\n", __FUNCTION__);
    if (gBackupState != NULL) {
       return RPCIN_SETRETVALS(data, "Quiesce operation already in progress.",
@@ -936,12 +1071,26 @@ VmBackupStart(RpcInData *data)
       if (StrUtil_GetNextIntToken(&generateManifests, &index, data->args, " ")) {
          gBackupState->generateManifests = generateManifests;
       }
-      gBackupState->quiesceApps = TRUE;
-      gBackupState->quiesceFS = TRUE;
-      gBackupState->allowHWProvider = TRUE;
-      gBackupState->execScripts = TRUE;
-      gBackupState->scriptArg = NULL;
-      gBackupState->timeout = 0;
+      gBackupState->quiesceApps = VMBACKUP_CONFIG_GET_BOOL(ctx->config,
+                                                           "quiesceApps",
+                                                           TRUE);
+      gBackupState->quiesceFS = VMBACKUP_CONFIG_GET_BOOL(ctx->config,
+                                                         "quiesceFS",
+                                                         TRUE);
+      gBackupState->allowHWProvider = VMBACKUP_CONFIG_GET_BOOL(ctx->config,
+                                                             "allowHWProvider",
+                                                             TRUE);
+      gBackupState->execScripts = VMBACKUP_CONFIG_GET_BOOL(ctx->config,
+                                                           "execScripts",
+                                                           TRUE);
+      gBackupState->scriptArg = VMBACKUP_CONFIG_GET_STR(ctx->config,
+                                                        "scriptArg",
+                                                        NULL);
+      gBackupState->timeout = VMBACKUP_CONFIG_GET_INT(ctx->config,
+                                                      "timeout", 0);
+      gBackupState->vssUseDefault = VMBACKUP_CONFIG_GET_BOOL(ctx->config,
+                                                             "vssUseDefault",
+                                                             TRUE);
 
       /* get volume uuids if provided */
       if (data->args[index] != '\0') {
@@ -949,7 +1098,9 @@ VmBackupStart(RpcInData *data)
                                            data->argsSize - index);
       }
    }
-   return VmBackupStartCommon(data, FALSE);
+   return VmBackupStartCommon(data, VMBACKUP_CONFIG_GET_BOOL(ctx->config,
+                                                             "forceQuiesce",
+                                                             FALSE));
 }
 
 
@@ -987,8 +1138,10 @@ VmBackupStart(RpcInData *data)
 static gboolean
 VmBackupStartWithOpts(RpcInData *data)
 {
+   ToolsAppCtx *ctx = data->appCtx;
    GuestQuiesceParams *params;
-   GuestQuiesceParamsV1 *paramsV1;
+   GuestQuiesceParamsV1 *paramsV1 = NULL;
+   GuestQuiesceParamsV2 *paramsV2;
    gboolean retval;
 
    g_debug("*** %s\n", __FUNCTION__);
@@ -997,24 +1150,57 @@ VmBackupStartWithOpts(RpcInData *data)
                               FALSE);
    }
    params = (GuestQuiesceParams *)data->args;
+
+#if defined(_WIN32)
+   if (params->ver != GUESTQUIESCEPARAMS_V1 &&
+       params->ver != GUESTQUIESCEPARAMS_V2) {
+      g_warning("%s: Incompatible quiesce parameter version. \n", __FUNCTION__);
+      return RPCIN_SETRETVALS(data, "Incompatible quiesce parameter version",
+                              FALSE);
+   }
+#else
    if (params->ver != GUESTQUIESCEPARAMS_V1) {
       g_warning("%s: Incompatible quiesce parameter version. \n", __FUNCTION__);
       return RPCIN_SETRETVALS(data, "Incompatible quiesce parameter version",
                               FALSE);
    }
+#endif
+
    gBackupState = g_new0(VmBackupState, 1);
-   paramsV1 = params->GuestQuiesceParams_u.guestQuiesceParamsV1;
-   gBackupState->generateManifests = paramsV1->createManifest;
-   gBackupState->quiesceApps = paramsV1->quiesceApps;
-   gBackupState->quiesceFS = paramsV1->quiesceFS;
-   gBackupState->allowHWProvider = paramsV1->writableSnapshot;
-   gBackupState->execScripts = paramsV1->execScripts;
-   gBackupState->scriptArg = g_strndup(paramsV1->scriptArg,
-                                       strlen(paramsV1->scriptArg));
-   gBackupState->timeout = paramsV1->timeout;
-   gBackupState->volumes = g_strndup(paramsV1->diskUuids,
-                                     strlen(paramsV1->diskUuids));
-   retval = VmBackupStartCommon(data, TRUE);
+
+   if (params->ver == GUESTQUIESCEPARAMS_V1) {
+      paramsV1 = params->GuestQuiesceParams_u.guestQuiesceParamsV1;
+      gBackupState->vssUseDefault = VMBACKUP_CONFIG_GET_BOOL(ctx->config,
+                                                             "vssUseDefault",
+                                                             TRUE);
+   } else if (params->ver == GUESTQUIESCEPARAMS_V2) {
+      paramsV2 = params->GuestQuiesceParams_u.guestQuiesceParamsV2;
+      paramsV1 = &paramsV2->paramsV1;
+      gBackupState->vssBackupContext = paramsV2->vssBackupContext;
+      gBackupState->vssBackupType = paramsV2->vssBackupType;
+      gBackupState->vssBootableSystemState = paramsV2->vssBootableSystemState;
+      gBackupState->vssPartialFileSupport = paramsV2->vssPartialFileSupport;
+      gBackupState->vssUseDefault = VMBACKUP_CONFIG_GET_BOOL(ctx->config,
+                                                             "vssUseDefault",
+                                                             FALSE);
+   }
+
+   if (paramsV1 != NULL) {
+      gBackupState->generateManifests = paramsV1->createManifest;
+      gBackupState->quiesceApps = paramsV1->quiesceApps;
+      gBackupState->quiesceFS = paramsV1->quiesceFS;
+      gBackupState->allowHWProvider = paramsV1->writableSnapshot;
+      gBackupState->execScripts = paramsV1->execScripts;
+      gBackupState->scriptArg = g_strndup(paramsV1->scriptArg,
+                                          strlen(paramsV1->scriptArg));
+      gBackupState->timeout = paramsV1->timeout;
+      gBackupState->volumes = g_strndup(paramsV1->diskUuids,
+                                        strlen(paramsV1->diskUuids));
+   }
+
+   retval = VmBackupStartCommon(data, VMBACKUP_CONFIG_GET_BOOL(ctx->config,
+                                                               "forceQuiesce",
+                                                               TRUE));
    return retval;
 }
 
@@ -1084,6 +1270,45 @@ VmBackupSnapshotDone(RpcInData *data)
 }
 
 
+/**
+ * Notifies the completer to complete the provider process
+ * machine in the COMPLETE_WAIT state.
+ *
+ * @param[in]  data     RPC data.
+ *
+ * @return TRUE
+ */
+
+static gboolean
+VmBackupSnapshotCompleted(RpcInData *data)
+{
+   g_debug("*** %s\n", __FUNCTION__);
+
+   if (gBackupState == NULL ||
+       gBackupState->completer == NULL) {
+      return RPCIN_SETRETVALS(data, "Error: no quiesce complete in progress",
+                              FALSE);
+   } else if (gBackupState->machineState != VMBACKUP_MSTATE_COMPLETE_WAIT) {
+      g_warning("Error: unexpected state for snapshot complete message: %s",
+                VmBackupGetStateName(gBackupState->machineState));
+      return RPCIN_SETRETVALS(data,
+                              "Error: unexpected state for complete message.",
+                              FALSE);
+   } else {
+      if (!gBackupState->completer->snapshotCompleted(gBackupState,
+             gBackupState->completer->clientData)) {
+         VmBackup_SendEvent(VMBACKUP_EVENT_REQUESTOR_ERROR,
+                            VMBACKUP_SYNC_ERROR,
+                            "Error when notifying the sync completer.");
+         if (VmBackupOnError()) {
+            VmBackupFinalize();
+         }
+      }
+      return RPCIN_SETRETVALS(data, "", TRUE);
+   }
+}
+
+
 /**
  * Prints some information about the plugin's state to the log.
  *
@@ -1108,9 +1333,9 @@ VmBackupDumpState(gpointer src,
 
 
 /**
- * Reset callback. Currently does nothing.
+ * Reset callback.
  *
- * @param[in]  src      The source object.
+ * @param[in]  src      The source object.  Unused.
  * @param[in]  ctx      Unused.
  * @param[in]  data     Unused.
  */
@@ -1120,6 +1345,7 @@ VmBackupReset(gpointer src,
               ToolsAppCtx *ctx,
               gpointer data)
 {
+   VmBackup_SyncDriverReset();
 }
 
 
@@ -1167,6 +1393,8 @@ ToolsOnLoad(ToolsAppCtx *ctx)
       { VMBACKUP_PROTOCOL_START_WITH_OPTS, VmBackupStartWithOpts, NULL,
                     xdr_GuestQuiesceParams, NULL, sizeof (GuestQuiesceParams) },
       { VMBACKUP_PROTOCOL_ABORT, VmBackupAbort, NULL, NULL, NULL, 0 },
+      { VMBACKUP_PROTOCOL_SNAPSHOT_COMPLETED, VmBackupSnapshotCompleted, NULL,
+                    NULL, NULL, 0 },
       { VMBACKUP_PROTOCOL_SNAPSHOT_DONE, VmBackupSnapshotDone, NULL, NULL, NULL, 0 }
    };
    ToolsPluginSignalCb sigs[] = {
